package cz.jzitnik.game.ui;

import org.jline.terminal.MouseEvent;
import org.jline.terminal.Terminal;
import cz.jzitnik.game.Game;
import cz.jzitnik.game.sprites.ui.Font.*;
import cz.jzitnik.tui.ScreenRenderer;
import cz.jzitnik.tui.utils.Menu;

public class Escape {
    private Game game;
    private Menu menu;

    public Escape(Game game) {
        this.game = game;
        this.menu = new Menu(game, "2DCraft", new String[] { "Continue", "Options", "Save and exit" },
                this::onButtonClick);
    }

    public void reset() {
        menu.reset();
    }

    public void render(StringBuilder buffer, Terminal terminal) {
        menu.render(buffer, terminal);
    }

    public void mouse(MouseEvent mouseEvent, Terminal terminal, ScreenRenderer screenRenderer) {
        menu.handleMouse(mouseEvent, terminal, screenRenderer);
    }

    private void onButtonClick(int index, ScreenRenderer screenRenderer) {
        switch (index) {
            case 0:
                game.setWindow(Window.WORLD);
                break;
            case 1:
                game.setWindow(Window.OPTIONS);
                break;
            case 2:
                game.setWindow(Window.SAVE_EXIT);
                screenRenderer.render(game);
                game.getGameStates().dependencies.gameSaver.save(game);
                game.setWindow(Window.SAVED);
                screenRenderer.render(game);
                System.exit(0);
                break;
        }
        screenRenderer.render(game);
    }

    public void renderSave(StringBuilder buffer, Terminal terminal) {
        var font = game.getGameStates().dependencies.font;
        var savingText = font.line(terminal, "Saving", Size.LARGE, Align.CENTER);

        int top = (terminal.getHeight() - savingText.getHeight()) / 2;

        buffer.append("\n".repeat(top));
        buffer.append(savingText.getData());
    }

    public void renderSaved(StringBuilder buffer, Terminal terminal) {
        var font = game.getGameStates().dependencies.font;
        var savingText = font.line(terminal, "Saved", Size.LARGE, Align.CENTER);

        int top = (terminal.getHeight() - savingText.getHeight()) / 2;

        buffer.append("\n".repeat(top));
        buffer.append(savingText.getData());
    }
}
